#!/bin/bash
#   Building Script
#
#	Copyright (C) 1991-1999 Theorist Interactive, Inc.
#	All Rights Reserved.  Proprietary and Confidential information.

# The print usage routine
function PrintUsageMessage
{  
	echo "Usage: build.sh <type> <product> <language>"
	echo "    <type>     : type of desired build: Debug or Release. default: Release"
	echo "    <product>  : product to build: OverComp, Theorist, Netscape or Viewer. default: all"
	echo "    <language> : language of the product. all or <empty> = english"
	echo ""
	echo "Platform and machine for build are autodetected:"
	echo ""
	exit -1
}

# makefiles concatenation routine.
# First parameter is makefile and second - object root. Third and forth are paltform and processor
function CreateMakefile
{
	makefile=$1
	objroot=$2
	platform=$3
	processor=$4
	product=$5

# add the starting portion first
	cat makefile.start > $makefile

# now add collected information in the makefile
    echo "### Script generated" >> $makefile
	echo "OSNAME       = $platform" >> $makefile
	echo "MACHNAME     = $processor" >> $makefile
	echo "RELEASE      = $TYPE" >> $makefile
	echo "BINROOT      = $BINROOT" >> $makefile
	echo "SRCROOT      = $SRCROOT" >> $makefile
	echo "OBJROOT      = $objroot" >> $makefile
	echo "TARGET       = $product" >> $makefile
	echo "STARTERS_DIR = $STARTERS_DIR" >> $makefile
	echo "" >> $makefile
	
# add default targets	
	echo "default : $product" >> $makefile
	echo "all : $product" >> $makefile
	echo "" >> $makefile

# add type specific part
	if [ -f makefile.$TYPE ]; then
		cat makefile.$TYPE >> $makefile
	fi

# now add platform specific part. if exists
  if [ -f makefile.$platform ]; then
		cat makefile.$platform >> $makefile
	fi

# add a processor specific part. if exists
	if [ -f makefile.$processor ]; then
		cat makefile.$processor >> $makefile
	fi

#add language part
	if [ "$LANGUAGE" == "all" ]; then
		echo "language: international" >> $makefile
	else
		echo "language: english" >> $makefile
	fi

	cat makefile.multi >> $makefile

# add the product part
	cat makefile.$product >> $makefile
	
# add common part
	cat makefile.common >> $makefile
	
# add the ending script
	if [ -f makefile.end ]; then
		cat makefile.end >> $makefile
	fi
}


##### Main start
# some system variables need resetting

# first set production defaults
SRCROOT=".."
BINROOT="../bin"
OBJROOT="objs"
LANGUAGE=$3

EXIT_STATUS=0

# if exist include 'default.sh' if it has some variables changed
if [ -f ./default.sh ]; then
  source ./default.sh
fi

# set the appropriate type of build
if [ "$1" == "" ]; then
	TYPE="Release"
else
	TYPE=$1
fi

if [ "$TYPE" != "Debug" ] && [ "$TYPE" != "Release" ]; then
	echo "Wrong type of build specified. Use: Release or Debug"
	PrintUsageMessage
fi

# determine the product(s) to build with 'product.sh'
PRODNAME=$2
source ./product.sh
if [ $EXIT_STATUS -ne 0 ]; then
	echo "Wrong product name specified. Use: all, OverGen, OverComp, OverEdit, Theorist, Viewer, Helper, Netscape7"
	PrintUsageMessage
fi

# detect platform and machine with 'platform.sh'
source ./platform.sh

## so everything is collected now.
# create necessary directories

# create binary root dirctory
pushd . > /dev/null
mkdir -p $BINROOT
cd $BINROOT
BINROOT=`pwd -P`
popd > /dev/null

# go to source directory to check its existence and obtain its full path
pushd $SRCROOT > /dev/null
if [ $? -ne 0 ]; then
	echo "Source directory $SRCROOT doesn't exist !"
	exit -1
fi
SRCROOT=`pwd -P`
popd > /dev/null

# print the collected information
echo "Building $TYPE version of $PRODNAME product(s) for $OSNAME running on $MACHNAME"
echo "  Source root     : $SRCROOT"
echo "  Binaries root   : $BINROOT"
echo "  Objectives root : $OBJROOT"
echo "  Starters root   : $STARTERS_DIR"
echo "  Language chosen : $LANGUAGE"
echo ""

### Now we are ready to start building the binary tree
echo "Creating the binary tree..."
platform=$OSNAME
processor=$MACHNAME

# male all necessary subdirectories in each product directory
for product in $PRODPREFIX
do
	mkdir -p $BINROOT/$product
 	mkdir -p $BINROOT/$product/resgen
	mkdir -p $BINROOT/$product/$OBJROOT
 	mkdir -p $BINROOT/$product/langs

	CreateMakefile $BINROOT/$product/Makefile $OBJROOT $platform $processor $product	  	
done

echo "Directory tree created."
# all the directories and files are created
echo "=> Start of make process... <="
echo "$platform-$processor" > $BINROOT/platform.info
OLD_LANG=$LANG
export LANG=en_US

for product in $PRODPREFIX
do
	pushd $BINROOT/$product > /dev/null
	make
 	MAKE_RES=$?
	ln -sf ../../BuildSrcs/CrashPage > /dev/null 2>/dev/null
	popd > /dev/null
	if [ $MAKE_RES != 0 ]; then
		break
	fi
done
if [ $MAKE_RES != 0 ]; then
	echo "Error: make process reported error ($MAKE_RES)"
	exit -1
fi

export LANG=$OLD_LANG
# That's all !
echo "Done."
